/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IPath;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.XMLMemento;

import eu.etaxonomy.cdm.database.CdmDataSource;
import eu.etaxonomy.cdm.database.CdmPersistentDataSource;
import eu.etaxonomy.cdm.database.DataSourceNotFoundException;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

public class CdmDataSourceRepository{
	private static final Logger logger = Logger
			.getLogger(CdmDataSourceRepository.class);

	public static final String TAG_DATASOURCE = "tagDataSource";
	private static final String CURRENT_DATASOURCE = "currentDataSource";
	private static final String DEFAULT_DATASOURCE_STATE_FILENAME = "datasource.xml";
	private static final String DEFAULT_DATASOURCE_NAME = "cdm";
	
	private static ICdmDataSource currentDataSource;
	private static XMLMemento memento;
	private static String lastUsedDataSourceName;
	
	public static String getLastUsedDataSourceName(){
		if(lastUsedDataSourceName == null){
			memento = readMemento();		
			lastUsedDataSourceName = memento != null ? memento.getString(CURRENT_DATASOURCE) : DEFAULT_DATASOURCE_NAME;
		}
		return lastUsedDataSourceName;
	}

	public static boolean delete(CdmPersistentDataSource dataSource) {
		CdmPersistentDataSource.delete(dataSource);
		return true;
	}

	
	public static List<ICdmDataSource> getAll() {
		List<ICdmDataSource> dataSources = new ArrayList<ICdmDataSource>();
		
		for(ICdmDataSource dataSource : CdmPersistentDataSource.getAllDataSources()){
			try {
				dataSources.add(CdmPersistentDataSource.NewInstance(dataSource.getName()));
			} catch (DataSourceNotFoundException e) {
				logger.error("Could not find dataSource", e);
			}
		}
		
		// TODO sort by database name
		
		return dataSources;
	}
	
	public static ICdmDataSource getDataSource(String name){
		
		for(ICdmDataSource dataSource : CdmPersistentDataSource.getAllDataSources()){
			try {
				if(dataSource.getName().equals(name)){
					return CdmPersistentDataSource.NewInstance(dataSource.getName());
				}
			} catch (DataSourceNotFoundException e) {
				logger.error("Could not find dataSource", e);
			}
		}
		
		return null;
	}
		
	/**
	 * 
	 * @return
	 */
	public static ICdmDataSource getCurrentDataSource() {
		if (currentDataSource == null) {
			try {
				currentDataSource = CdmPersistentDataSource.NewInstance(getLastUsedDataSourceName());
			} catch (DataSourceNotFoundException e) {
				// fallback creates a new default
				ICdmDataSource h2DataSource = CdmDataSource.NewH2EmbeddedInstance(
						DEFAULT_DATASOURCE_NAME, "sa", "", PreferencesUtil.getPreferredNomenclaturalCode());
				save(h2DataSource.getName(), h2DataSource);
				setCurrentDataSource(h2DataSource);
			}
		}
		return currentDataSource;
	}
	
	/**
	 * 
	 * @param dataSource
	 * @return
	 */
	public static boolean setCurrentDataSource(ICdmDataSource dataSource) {
		currentDataSource = dataSource;
		NomenclaturalCode dataSourceNomenclaturalCode = dataSource.getNomenclaturalCode();
		NomenclaturalCode applicationNomenclaturalCode = PreferencesUtil.getPreferredNomenclaturalCode();
		
		if( dataSourceNomenclaturalCode != null && ! dataSourceNomenclaturalCode.equals(applicationNomenclaturalCode)){
			PreferencesUtil.setPreferredNomenclaturalCode(dataSourceNomenclaturalCode);
			StoreUtil.informationDialog("Nomenclatural Code Change", "The Datasource that was just " +
					"loaded has a different nomenclatural code than the one stored in Preferences. " +
					"The nomenclatural code was changed in the application.");
		}
		return true;
	}	
		
	/**
	 * 
	 * @param dataSource
	 * @return
	 */
	public static boolean changeDataSource(final ICdmDataSource dataSource) {

		// TODO Close all open editors before showing progress monitor
		if (!StoreUtil.closeAll()) {
			// User has canceled operation   
			return false;
		}
		
		memento.putString(CURRENT_DATASOURCE, dataSource.getName());
		saveMementoToFile(memento);
		PlatformUI.getWorkbench().restart();
	
		return true;
	}

	/**
	 * @param dataSource
	 * @return
	 */
	public static CdmPersistentDataSource save(String strDataSourceName, ICdmDataSource dataSource) {
		return CdmPersistentDataSource.save(strDataSourceName, dataSource);
	}
	
	public static CdmPersistentDataSource update(String strDataSourceName, ICdmDataSource dataSource){
		try {
			return CdmPersistentDataSource.update(strDataSourceName, dataSource);
		} catch (Exception e) {
			logger.error("Error updating datasource", e);
		}
		return null;
	}
	
	/*********************************************************
	 * Memento Handling										 *
	 *********************************************************/
	
	/*
	 * Answer the workbench state file.
	 */
	private static File getDataSourceStateFile() {
		IPath path = TaxeditorStorePlugin.getDefault().getStateLocation();
		if (path == null) {
			return null;
		}
		path = path.append(DEFAULT_DATASOURCE_STATE_FILENAME);
		return path.toFile();
	}
	
	private static XMLMemento readMemento(){
		File stateFile = getDataSourceStateFile();
		FileInputStream input;
		try {
			input = new FileInputStream(stateFile);
			BufferedReader reader = new BufferedReader(
					new InputStreamReader(input, "utf-8")); //$NON-NLS-1$
			return XMLMemento.createReadRoot(reader);
		} catch (FileNotFoundException e) {
			// first start the file is not there
			return initializeMemento();
		} catch (WorkbenchException e) {
			logger.error("Exception while reading the memento", e);
		} catch (UnsupportedEncodingException e) {
			logger.error("Exception while reading the memento", e);
		}

		return null;
	}
	
	/**
	 * 
	 */
	private static XMLMemento initializeMemento() {
		
		XMLMemento memento = XMLMemento.createWriteRoot(TAG_DATASOURCE);
		memento.putString(CURRENT_DATASOURCE, DEFAULT_DATASOURCE_NAME);
		saveMementoToFile(memento);
		
		return readMemento();
	}

	/*
	 * Save the workbench UI in a persistence file.
	 */
	private static boolean saveMementoToFile(XMLMemento memento) {
		// Save it to a file.
		// XXX: nobody currently checks the return value of this method.
		File stateFile = getDataSourceStateFile();
		if (stateFile == null) {
			return false;
		}
		try {
			FileOutputStream stream = new FileOutputStream(stateFile);
			OutputStreamWriter writer = new OutputStreamWriter(stream, "utf-8"); //$NON-NLS-1$
			memento.save(writer);
			writer.close();
		} catch (IOException e) {
			stateFile.delete();
			logger.error("Could not save datasource state");
			return false;
		}

		// Success !
		return true;
	}	
}