// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs.filteredSelection;

import java.text.Collator;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.dialogs.FilteredItemsSelectionDialog;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public abstract class AbstractFilteredCdmResourceSelectionDialog<T extends CdmBase> extends
		FilteredItemsSelectionDialog {
	private static final Logger logger = Logger
			.getLogger(AbstractFilteredCdmResourceSelectionDialog.class);


	protected List<UuidAndTitleCache<T>> model;
	private Set<T> transientCdmObjects = new HashSet<T>();
	private String settings;	
	
	/**
	 * @param shell
	 */
	protected AbstractFilteredCdmResourceSelectionDialog(Shell shell, String title, boolean multi, String settings, T cdmObject) {
		super(shell, multi);
		setTitle(title);
		setMessage("Use * for wildcard, or ? to see all entries");
		this.settings = settings;
		
		initModel();
		
		if (cdmObject != null) {
			// If agent is not persistent, add it to the model
			if (isObjectTransient(cdmObject)) {
				addObjectToModel(cdmObject);
			}
			setInitialPattern(getTitleCache(cdmObject));
		}
		
		ILabelProvider labelProvider = new FilteredCdmResourceLabelProvider();
		setListLabelProvider(labelProvider);
		setDetailsLabelProvider(labelProvider);
		
//		setSelectionHistory(new ResourceSelectionHistory());
	}
	
	/**
	 * @param dialog
	 * @return
	 */
	protected static <TYPE extends CdmBase> TYPE getSelectionFromDialog(AbstractFilteredCdmResourceSelectionDialog<TYPE> dialog) {
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		
		UUID uuid = dialog.getSelectedUuidAndTitleCache().getUuid();
		if(uuid == null){
			return null;
		}	
		return dialog.getCdmObjectByUuid(uuid);
	}
	
	/**
	 * Check if object was created during the life of this dialog. If not,
	 * retrieve it from the CdmStore.
	 * 
	 * @param cdmUuid
	 * @return
	 */
	protected T getCdmObjectByUuid(UUID cdmUuid) {
		for (T cdmObject : transientCdmObjects) {
			if (cdmObject.getUuid().equals(cdmUuid)) {
				return cdmObject;
			}
		}
		return getPersistentObject(cdmUuid);
	}
	
	/**
	 * @param cdmUuid
	 * @return
	 */
	abstract protected T getPersistentObject(UUID uuid);

	/**
	 * @param cdmObject
	 */
	protected void addObjectToModel(T cdmObject) {
		model.add(new UuidAndTitleCache(cdmObject.getClass(), cdmObject.getUuid(), getTitleCache(cdmObject)));
		transientCdmObjects.add(cdmObject);
	}

	/**
	 * @param cdmObject
	 * @return
	 */
	protected boolean isObjectTransient(T cdmObject) {
		return (getPersistentObject(cdmObject.getUuid()) == null);
	}

	/**
	 * @param cdmObject
	 * @return
	 */
	private String getTitleCache(T cdmObject) {
		if (cdmObject instanceof IIdentifiableEntity) {
			return ((IIdentifiableEntity) cdmObject).getTitleCache();			
		}
		return null;
	}

	/**
	 * 
	 */
	abstract protected void initModel();

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createFilter()
	 */
	@Override
	protected ItemsFilter createFilter() {
		return new ItemsFilter() {

			@Override
			public boolean isConsistentItem(Object item) {
				return false;
			}

			@Override
			public boolean matchItem(Object item) {
				String text = null;
				if(item instanceof UuidAndTitleCache){
					text = ((UuidAndTitleCache) item).getTitleCache();
				}else if(item instanceof String){
					text = (String) item;
				}
				if (text != null) {
					logger.info(text + " " + matches(text));// + " " + model.size());
				} else {
					logger.info("null");
				}
				return text != null ? matches(text) : false;
			}
			
		};
	}
	

	/**
	 * Set the filter input to the Agent's title cache 
	 * 
	 * @param teamOrPerson
	 */
	protected void setPattern(T cdmObject) {
		((Text) getPatternControl()).setText(getTitleCache(cdmObject));
	}
	
	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#fillContentProvider(org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.AbstractContentProvider, org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.ItemsFilter, org.eclipse.core.runtime.IProgressMonitor)
	*/
	@Override
	protected void fillContentProvider(AbstractContentProvider contentProvider,
		ItemsFilter itemsFilter, IProgressMonitor progressMonitor)
		throws CoreException {
		try {
			progressMonitor.beginTask("Looking for names", model.size()); // Why no progress monitor?
			for(UuidAndTitleCache<T> element : model){
				contentProvider.add(element, itemsFilter);
				if (progressMonitor.isCanceled()) {
					throw new OperationCanceledException();
				}
				progressMonitor.worked(1);
			}
		}
		finally {
			progressMonitor.done();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getDialogSettings()
	 */
	@Override
	protected IDialogSettings getDialogSettings() {
		IDialogSettings settings = TaxeditorStorePlugin.getDefault().getDialogSettings().getSection(getSettings());

		if (settings == null) {
			settings = TaxeditorStorePlugin.getDefault().getDialogSettings().addNewSection(getSettings());
		}
		return settings;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getElementName(java.lang.Object)
	 */
	@Override
	public String getElementName(Object item) {
		return ((UuidAndTitleCache) item).getTitleCache();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getItemsComparator()
	 */
	@Override
	protected Comparator getItemsComparator() {
		return new Comparator<UuidAndTitleCache>() {
			public int compare(UuidAndTitleCache entity1,
					UuidAndTitleCache entity2) {
				Collator collator = Collator.getInstance();
				return collator.compare(entity1.getTitleCache(), entity2.getTitleCache());
			}
		};
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#validateItem(java.lang.Object)
	 */
	@Override
	protected IStatus validateItem(Object item) {
		return Status.OK_STATUS;
	}
	
	/**
	* @return
	*/
	protected UuidAndTitleCache getSelectedUuidAndTitleCache() {
		Object[] result = getResult();
		return result[0] == null ? null : (UuidAndTitleCache) result[0];
	}
	
	public String getSettings()  {
		if(settings == null){
			throw new IllegalStateException("No SETTINGS set.");
		}
		return settings;
	}
	
	/**
	 * 
	 * @author n.hoffmann
	 * @created Oct 19, 2009
	 * @version 1.0
	 */
	private class ResourceSelectionHistory extends SelectionHistory {
	    /*
	    * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.SelectionHistory#restoreItemFromMemento(org.eclipse.ui.IMemento)
	   	*/
		protected Object restoreItemFromMemento(IMemento element) {
			return element.getString("resource"); //$NON-NLS-1$
	  	}
	  	/*
	  	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.SelectionHistory#storeItemToMemento(java.lang.Object,
	  	 *      org.eclipse.ui.IMemento)
	  	 */
		protected void storeItemToMemento(Object item, IMemento element) {
			element.putString("resource", item.toString()); //$NON-NLS-1$
		}
	}
	
	public class FilteredCdmResourceLabelProvider extends LabelProvider {
		public String getText(Object element) {
			if (element == null) {
				return null;
			}
			return ((UuidAndTitleCache) element).getTitleCache();
		}			
	};

}
