// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.undo.WorkspaceUndoUtil;
import org.eclipse.ui.themes.ITheme;
import org.eclipse.ui.themes.IThemeManager;

import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author n.hoffmann
 * @created 11.05.2009
 * @version 1.0
 */
public abstract class AbstractUtility {
	private static final Logger logger = Logger.getLogger(AbstractUtility.class);
	
	protected static IStatusLineManager statusLineManager;
	
	/**
	 * @return
	 */
	public static Shell getShell() {
		return TaxeditorStorePlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
	}
	
	public static IWorkbenchPage getActivePage(){
		return TaxeditorStorePlugin.getDefault().getWorkbench()
			.getActiveWorkbenchWindow().getActivePage();
	}
	
	public static IViewPart getView(String id) throws PartInitException{
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(id);
	}
	
	public static boolean closeAll() {
		return getActivePage().closeAllEditors(true);
	}
	
	/**
	 * 
	 * @param api
	 * @return
	 */
	public static Object getService(Class api){
		return TaxeditorStorePlugin.getDefault().getWorkbench().getService(api);
	}
	
	public static ITheme getCurrentTheme(){
		IThemeManager themeManager = TaxeditorStorePlugin.getDefault().getWorkbench().getThemeManager();
		return themeManager.getCurrentTheme();
	}
	
	/**
	 * Fonts registered to the plugin may be obtained with the Eclipse themeing functionality.
	 * Thus fonts are chooseable by the user via Preferences->General->Appearance->Colors and Fonts
	 * 
	 * @return the FontRegistry for the current theme
	 */
	public static FontRegistry getFontRegistry(){
		return getCurrentTheme().getFontRegistry();
	}
	
	/**
	 * Color registered to the plugin may be obtained with the Eclipse themeing functionality.
	 * Thus colors are editable by the user via Preferences->General->Appearance->Colors and Fonts
	 * 
	 * @return the ColorRegistry for the current theme
	 */
	public static ColorRegistry getColorRegistry(){
		return getCurrentTheme().getColorRegistry();
	}
	
	/**
	 * Open a message box that informs the user about unimplemented functionality.
	 * This method is for developer convenience.
	 */
	public static void notImplementedMessage(){
		warningDialog("Not yet implemented", "This functionality is not yet implemented.");
	}
	
	public static void informationDialog(String title, String message){
		MessageDialog.openInformation(getShell(), title, message);
	}
	
	public static void warningDialog(String title, String message){
		MessageDialog.openWarning(getShell(), title, message);
	}
	
	public static void errorDialog(String title, String message){
		MessageDialog.openError(getShell(), title, message);
	}
	
	public static IStatus executeOperation(IUndoableOperation operation){
		if(getOperationHistory() == null){
			throw new IllegalArgumentException("There is no operation history for this context");
		}
		
		// Start the main progress monitor.
        IProgressMonitor newMonitor = startMainMonitor(getMonitor(),operation.getLabel(), 100);

		// Check whether operation was canceled and do some steps.
        workedChecked(newMonitor, 10);

        try {
			IStatus status = getOperationHistory().execute(operation, newMonitor,
					WorkspaceUndoUtil.getUIInfoAdapter(getShell()));

			// Check whether operation was canceled and do some steps.
			workedChecked(newMonitor, 30);

	        String statusString = status.equals(Status.OK_STATUS) ? "completed" : "cancelled";
			setStatusLine(operation.getLabel() + " " + statusString + ".");

	        return status;
		} catch (ExecutionException e) {
			logger.error("Error executing operation: " + operation.getLabel(), e);
		}
        finally {
        	
        	// Stop the progress monitor.
            newMonitor.done();
        }
		return null;
	}
	
	public static IOperationHistory getOperationHistory(){
		return TaxeditorStorePlugin.getDefault().getWorkbench().
							getOperationSupport().getOperationHistory();
	}
	
	public static void setStatusLine(String message) {
		statusLineManager.setMessage(message);
	}
	
	protected static IProgressMonitor getMonitor() {
		statusLineManager.setCancelEnabled(false);
		return statusLineManager.getProgressMonitor();
	}
	
	/**
	 * Starts either the given {@link IProgressMonitor} if it's not <code>null</code> or a new {@link NullProgressMonitor}.
	 * 
	 * @param progressMonitor The {@link IProgressMonitor} or <code>null</code> if no progress should be reported.
	 * @param taskName The name of the main task.
	 * @param steps The number of steps this task is subdivided into.
	 * @return The {@link IProgressMonitor}.
	 */
    public static IProgressMonitor startMainMonitor(IProgressMonitor progressMonitor, String taskName, int steps) {
        IProgressMonitor newMonitor = progressMonitor;
        if (newMonitor == null) {
            newMonitor = new NullProgressMonitor();
        }
        newMonitor.beginTask(taskName == null ? "" : taskName, steps);
        newMonitor.subTask(" ");
        return newMonitor;
    }

    /**
     * Creates a {@link SubProgressMonitor} if the given {@link IProgressMonitor} is not <code>null</code> and not a {@link NullProgressMonitor}.
	 * @param progressMonitor The parent {@link IProgressMonitor} of the {@link SubProgressMonitor} to be created.
     * @param steps The number of steps this subtask is subdivided into. Must be a positive number and must not be {@link IProgressMonitor#UNKNOWN}.
     * @return The {@link IProgressMonitor}.
     */
    public static IProgressMonitor getSubProgressMonitor(IProgressMonitor progressMonitor, int steps) {
        if (progressMonitor == null) {
            return new NullProgressMonitor();
        }
        if (progressMonitor instanceof NullProgressMonitor) {
            return progressMonitor;
        }
        return new SubProgressMonitor(progressMonitor, steps, SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK);
    }
    
	/**
	 * Checks whether the user canceled this operation. If not canceled, the given number of steps are declared as done.
	 * @param newMonitor
	 * @return 
	 */
	public static void workedChecked(IProgressMonitor newMonitor, int steps) {
		// In case the progress monitor was canceled throw an exception.
		if (newMonitor.isCanceled()) {
			throw new OperationCanceledException();
		}
		// Otherwise declare this step as done.
		newMonitor.worked(steps);
	}


}
