/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IEditorInput;

import eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.impl.TaxonServiceConfiguratorImpl;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.ExtensionType;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IDefinedTerm;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.media.RightsTerm;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.cdm.strategy.match.DefaultMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.MatchException;
import eu.etaxonomy.cdm.strategy.match.MatchMode;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.store.VocabularyStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author p.ciardelli
 * @created 05.12.2008
 * @version 1.0
 * @author n.hoffmann
 */
public class PreferencesUtil {
	private static final Logger logger = Logger
			.getLogger(PreferencesUtil.class);

	public static final String PREFERRED_NOMENCLATURAL_CODE_KEY = "code";
	
	public static final String TAXON_SERVICE_CONFIGURATOR_TAXA = "taxonServiceConfigurator.doTaxa";
	public static final String TAXON_SERVICE_CONFIGURATOR_SYNONYMS = "taxonServiceConfigurator.doSynonyms";
	public static final String TAXON_SERVICE_CONFIGURATOR_NAMES = "taxonServiceConfigurator.doNamesWithoutTaxa";
	public static final String TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES = "taxonServiceConfigurator.doTaxaByCommonNames";
	public static final String TAXON_SERVICE_CONFIGURATOR_ORPHANED_TAXA = "taxonServiceConfigurator.doTaxaByOrphanedTaxa";
	
	public static final String SHOW_DATASOURCE_CHOOSER = "dataSourceChooser.show";

	public static final String PREFERRED_TERMS_CHANGE = "preferred_terms";

	public static final String HIDE_BULKEDITOR_INFO = "bulkeditorInfo.hide";

	public static final String EDIT_MARKER_TYPE_PREFIX = "editMarkerType";
	
	public static final String MATCH_STRATEGY_PREFIX = "matchStrategy.";
		
	/**
	 * 
	 * @return
	 */
	public static IPreferenceStore getPreferenceStore() {
		return TaxeditorStorePlugin.getDefault().getPreferenceStore();
	}
		
	/**
	 * 
	 * @param preferredCode
	 */
	public static void setPreferredNomenclaturalCode(NomenclaturalCode preferredCode) {
		getPreferenceStore().setValue(PREFERRED_NOMENCLATURAL_CODE_KEY, getPreferenceKey(preferredCode));
	}
		
	/**
	 * @return
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode() {
	
		for (NomenclaturalCode code : NomenclaturalCodeHelper.getAllCodes()) {
			String preferredCode = getPreferenceStore().getString(PREFERRED_NOMENCLATURAL_CODE_KEY);
			if (getPreferenceKey(code).equals(preferredCode)) {
				return code;
			}
		}
		return null;
	}
	
	/**
	 * The ability to edit marker types in the BulkEditor is associated with an input type, e.g. references.
	 * 
	 * @param input
	 * @param markerType
	 * @return
	 */
	public static boolean getEditMarkerTypePreference(IEditorInput input, MarkerType markerType) {
		return getPreferenceStore().getBoolean(getEditMarkerTypePrefKey(input, markerType)); 
	}

	public static void setEditMarkerTypePreference(IEditorInput input, MarkerType markerType, boolean edit) {
		getPreferenceStore().setValue(getEditMarkerTypePrefKey(input, markerType), edit); 
	}
	
	private static String getEditMarkerTypePrefKey(IEditorInput input, MarkerType markerType) {
		markerType = (MarkerType) HibernateProxyHelper.deproxy(markerType);
		return EDIT_MARKER_TYPE_PREFIX + "." + input.getClass().getName() + "." + markerType.getUuid().toString();
	}
	
	/**
	 * Get the match strategy for the given class that was stored in preferences
	 * or the default strategy if it was not stored in preferences
	 * 
	 * @param clazz
	 * @return
	 */
	public static IMatchStrategy getMatchStrategy(Class clazz){
		String className = clazz.getName();
		if(getPreferenceStore().getBoolean(MATCH_STRATEGY_PREFIX + className)){
			IMatchStrategy matchStrategy = getDefaultMatchStrategy(clazz);
			
			for(String fieldName : matchStrategy.getMatchFieldPropertyNames()){
				String matchModeName = getPreferenceStore().getString(getMatchStrategyFieldName(className, fieldName));
				MatchMode matchMode = MatchMode.valueOf(matchModeName);
				try {
					matchStrategy.setMatchMode(fieldName, matchMode);
				} catch (MatchException e) {
					logger.error(e);
					throw new RuntimeException(e);
				}
			}
			
			return matchStrategy;
		}
		return getDefaultMatchStrategy(clazz);
	}
	
	/**
	 * Stores a matchStrategy into the preference store.
	 * 
	 * @param clazz
	 */
	public static void setMatchStrategy(IMatchStrategy matchStrategy){
		String className = matchStrategy.getMatchClass().getName();
		getPreferenceStore().setValue(MATCH_STRATEGY_PREFIX + className, true);
		
		Set<String> matchFields = matchStrategy.getMatchFieldPropertyNames();
		
		for(String fieldName : matchFields){
			getPreferenceStore().setValue(getMatchStrategyFieldName(className, fieldName),
					matchStrategy.getMatchMode(fieldName).name());
		}
	}
	
	/**
	 * Helper method to create the preference property for a match field.
	 * 
	 * @param className
	 * @param fieldName
	 * @return
	 */
	private static String getMatchStrategyFieldName(String className, String fieldName){
		return MATCH_STRATEGY_PREFIX + className  + "." +  fieldName;
	}
	
	/**
	 * Returns the default match strategy for a given class.
	 * 
	 * @param clazz
	 * @return
	 */
	public static IMatchStrategy getDefaultMatchStrategy(Class clazz){
		return DefaultMatchStrategy.NewInstance(clazz);
	}
	
	/**
	 * Generic method to get term preferences for a term vocabulary
	 * 
	 * @param <T>
	 * @param initialTerms
	 * @return
	 */
	public static <T extends DefinedTermBase> List<T> getPreferredTerms(Set<T> initialTerms){
		
		// set default for non existent preferences
		defaultTerms(initialTerms);
				
		List<T> preferredTerms = new ArrayList<T>();
		
		for (T term : initialTerms){
			if(getPreferenceStore().getBoolean(getPreferenceKey((ICdmBase) term))){
				preferredTerms.add(term);
			}
		}
		
		return preferredTerms;		
	}
	
	private static <T extends DefinedTermBase> boolean defaultTerms(Set<T> initialTerms){
		// set default for non existen preferences
		for(T term : initialTerms){
			if(getPreferenceStore().getBoolean(term.getClass().getName())){
				break;
			}else{
				getPreferenceStore().setValue(term.getClass().getName(), true);
				setPreferredTerms(new ArrayList(initialTerms), initialTerms);
				break;
			}
		}
		return true;
	}
	
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<SpecimenTypeDesignationStatus> getPreferredSpecimenTypeDesignationStatus() { return getPreferredTerms(VocabularyStore.getSpecimenTypeDesignationStatus());}

	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<SpecimenTypeDesignationStatus> getPreferredNameTypeDesignationStatus() { return getPreferredTerms(VocabularyStore.getNameTypeDesignationStatus());}
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<TaxonRelationshipType> getPreferredTaxonRelationshipTypes() { return getPreferredTerms(VocabularyStore.getTaxonRelationshipTypes());}
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<Feature> getPreferredFeatures() { return getPreferredTerms(VocabularyStore.getFeatures());}

	/** @return a <code>List</code> containing user preferred <code>Rank</code> terms */
	public static List<Rank> getPreferredRanks() { return getPreferredTerms(VocabularyStore.getRanks());}
	
	/** @return a <code>List</code> containing user preferred  <code>PresenceAbsenceTermBase</code> terms */
	public static List<PresenceAbsenceTermBase<PresenceAbsenceTermBase<?>>> getPreferredPresenceAbsenceTerms(){ return getPreferredTerms(VocabularyStore.getPresenceAbsenceTerms());}
	
	/** @return a <code>List</code> containing user preferred  <code>NomenclaturalStatusType</code> terms */
	public static List<NomenclaturalStatusType> getPreferredNomenclaturalStatusTypes(){	return getPreferredTerms(VocabularyStore.getNomenclaturalStatusTypes());}
	
	/** @return a <code>List</code> containing user preferred  <code>NameRelationshipType</code> terms */
	public static List<NameRelationshipType> getPreferredNameRelationshipTypes(){ return getPreferredTerms(VocabularyStore.getNameRelationshipTypes());}
	
	/** @return a <code>List</code> containing user preferred  <code>Language</code> terms */
	public static List<Language> getPreferredLanguages() { return getPreferredTerms(VocabularyStore.getLanguages()); }
	
	/** @return a <code>List</code> containing user preferred  <code>MarkerType</code> terms */
	public static List<MarkerType> getPreferredMarkerTypes() { return getPreferredTerms(VocabularyStore.getNonTechnicalMarkerTypes()); }
	
	/** @return a <code>List</code> containing user preferred  <code>ExtensionType</code> terms */
	public static List<ExtensionType> getPreferredExtensionTypes() { return getPreferredTerms(VocabularyStore.getExtensionTypes()); }
	
	/** @return	a <code>List</code> containing user preferred  <code>RightsType</code> terms  */
	public static List<RightsTerm> getPreferredRightsTypes() { 	return getPreferredTerms(VocabularyStore.getRightsTypes());}
	
	
	/**
	 * Generic method to set term preferences
	 * 
	 * @param preferredTerms
	 * @param initialTerms
	 */
	public static <T extends DefinedTermBase> void setPreferredTerms(List<T> preferredTerms, Set<T> initialTerms){
		for(TermBase term : initialTerms){
			getPreferenceStore().setValue(getPreferenceKey(term), preferredTerms.contains(term));
		}
	}
	
	/**
	 * Construct a unique key using the CdmBase object's uuid
	 * 
	 * @param cdmBase
	 * @return
	 */
	private static String getPreferenceKey(ICdmBase cdmBase) {
		cdmBase = (ICdmBase) HibernateProxyHelper.deproxy(cdmBase);
		
		String key = cdmBase.getClass().getName()
				. concat(".")
				. concat(cdmBase.getUuid().toString());
		if (key.contains("javassist")) {
			logger.info("proxy");
		}
		return key;
	}

	/**
	 * Construct a unique key using the CdmBase object's uuid
	 * 
	 * @param cdmBase
	 * @return
	 */
	private static String getPreferenceKey(IDefinedTerm definedTerm) {
		definedTerm = (IDefinedTerm) HibernateProxyHelper.deproxy(definedTerm);
		String key = definedTerm.getClass().getName()
				. concat(".")
				. concat(definedTerm.getUuid().toString());
		if (key.contains("javassist")) {
			logger.info("proxy");
		}
		return key;
	}
	
	/**
	 * Retrieves search preferences from the preference store
	 * 
	 * @return an <code>ITaxonServiceConfigurator</code> to pass to search methods
	 */
	public static ITaxonServiceConfigurator getSearchConfigurator() {
		ITaxonServiceConfigurator configurator = initializeSearchConfigurator();

		configurator.setDoTaxa(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_TAXA));
		configurator.setDoSynonyms(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_SYNONYMS));
		configurator.setDoNamesWithoutTaxa(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_NAMES));
		configurator.setDoTaxaByCommonNames(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES));			
		configurator.setDoTaxaByOrphanedTaxa(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_ORPHANED_TAXA));

		return configurator;
	}
	
	/**
	 * create new preferences, setting all search options to true
	 * @return 
	 */
	public static ITaxonServiceConfigurator initializeSearchConfigurator(){
		ITaxonServiceConfigurator configurator = TaxonServiceConfiguratorImpl.NewInstance();
		
		configurator.setDoTaxa(true);
		configurator.setDoSynonyms(true);
		configurator.setDoNamesWithoutTaxa(true);
		configurator.setDoTaxaByCommonNames(true);
		configurator.setDoTaxaByOrphanedTaxa(true);
		
		// DEFAULT VALUES		
		// match mode default only
		configurator.setMatchMode(eu.etaxonomy.cdm.persistence.query.MatchMode.BEGINNING);
		// i don't know what happens to sec at the moment
		configurator.setSec(null);
		// we set page number and size here as this should always be unlimited
		configurator.setPageNumber(0);
		// TODO currently limit results to 1000
		configurator.setPageSize(1000);
		
		return configurator;
	}
	
	/**
	 * Store search preferences
	 * 
	 * @param configurator
	 */
	public static void setSearchConfigurator(ITaxonServiceConfigurator configurator){
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_TAXA, configurator.isDoTaxa());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_SYNONYMS, configurator.isDoSynonyms());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_NAMES, configurator.isDoNamesWithoutTaxa());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES, configurator.isDoTaxaByCommonNames());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_ORPHANED_TAXA, configurator.isDoTaxaByOrphanedTaxa());
	}

	/**
	 * @param class1
	 */
	public static void firePreferencesChanged(Class clazz) {
		getPreferenceStore().firePropertyChangeEvent(PREFERRED_TERMS_CHANGE, null, clazz);
	}
}
