// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.preference.menu;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author n.hoffmann
 * @created 12.06.2009
 * @version 1.0
 */
public abstract class AbstractMenuPreferences<T extends DefinedTermBase> extends PreferencePage implements
		IWorkbenchPreferencePage{

	private Button toggleButton;
	protected HashMap<DefinedTermBase<T>, Button> menuButtons;
	
	protected Button newButton;
	protected Button editButton;
	protected Button removeButton;
	
	protected Set<T> vocabulary;
	
	private List<T> preferedTerms;
	
	protected CheckboxTableViewer tableViewer;
	
	private boolean state = true;
	private boolean vocabularyIsEditable;
	
	/**
	 * Constructs a new menu preference page.
	 * 
	 * @param initialVocabulary 
	 * 			the initial vocabulary containing all possible entries for the menu
	 * @param preferedTerms 
	 * 			a List containing already preferred terms 
	 * @param title 
	 * 			the title of the page
	 * @param description 
	 * 			describing text for the preference page
	 * @param editable
	 * 			whether this vocabulary should be editable. Will render "New", "Edit", "Delete" buttons
	 * 			Handler for these buttons have to be overriding addEditButtonListeners()
	 */
	public AbstractMenuPreferences(Set<T> initialVocabulary, String title, String description, boolean editable) {
		super(title);
		vocabulary = initialVocabulary;
		preferedTerms = PreferencesUtil.getPreferredTerms(initialVocabulary);
		vocabularyIsEditable = editable;
		setDescription(description);
	}

	/**
	 * Create contents of the preference page
	 * @param parent
	 */
	@Override
	public Control createContents(Composite parent) {
		
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		container.setLayout(gridLayout);
	
		tableViewer = CheckboxTableViewer.newCheckList(container, SWT.NULL);
		GridData tableLayoutData = new GridData(SWT.FILL, SWT.FILL, true, false);
		tableLayoutData.heightHint = 300;
		tableViewer.getTable().setLayoutData(tableLayoutData);
		
		tableViewer.setContentProvider(new MenuPreferenceContentProvider());
		tableViewer.setLabelProvider(new MenuPreferenceLabelProvider());
			
		tableViewer.setInput(vocabulary);
		
		tableViewer.setCheckedElements(preferedTerms.toArray());
		
		Composite buttonContainer = new Composite(container, SWT.NULL);
		GridData buttonContainerLayoutData = new GridData();
		buttonContainerLayoutData.verticalAlignment = SWT.TOP;
		buttonContainer.setLayoutData(buttonContainerLayoutData);
		buttonContainer.setLayout(new GridLayout());
		
		if(vocabularyIsEditable) createEditButtons(buttonContainer);
		
		toggleButton = new Button(buttonContainer, SWT.PUSH);
		toggleButton.setText("Toggle");
		toggleButton.addSelectionListener(new SelectionAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				tableViewer.setAllChecked(state);
				state = state ? false : true;
			}
		});
		
		return container;
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.preference.AbstractMenuPreferences#createButtons(org.eclipse.swt.widgets.Composite)
	 */
	protected void createEditButtons(Composite buttonContainer) {
		newButton = new Button(buttonContainer, SWT.PUSH);
		newButton.setText("New");
		newButton.setLayoutData(new GridData(SWT.FILL));

		
		editButton = new Button(buttonContainer, SWT.PUSH);
		editButton.setText("Edit");
		editButton.setLayoutData(new GridData(SWT.FILL));

		
		removeButton = new Button(buttonContainer, SWT.PUSH);
		removeButton.setText("Remove");
		removeButton.setLayoutData(new GridData(SWT.FILL));

		
		addNewButtonListeners();
		addEditButtonListeners();
		addDeleteButtonListeners();
	}

	/**
	 * Implement this method in MenuPreference Pages where the vocabulary should be editable, editable flag 
	 * is set.
	 */
	protected void addNewButtonListeners() {
		newButton.addSelectionListener(new SelectionAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				super.widgetSelected(e);
				StoreUtil.notImplementedMessage();
			}
		});
	}
	
	/**
	 * Implement this method in MenuPreference Pages where the vocabulary should be editable, editable flag 
	 * is set.
	 */
	protected void addEditButtonListeners() {
		editButton.addSelectionListener(new SelectionAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				super.widgetSelected(e);
				StoreUtil.notImplementedMessage();
			}
		});
	}
	
	/**
	 * Implement this method in MenuPreference Pages where the vocabulary should be editable, editable flag 
	 * is set.
	 */
	protected void addDeleteButtonListeners() {
		removeButton.addSelectionListener(new SelectionAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				super.widgetSelected(e);
				StoreUtil.notImplementedMessage();
			}
		});
	}

	public void init(IWorkbench workbench) {
		setPreferenceStore(PreferencesUtil.getPreferenceStore());
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {
		tableViewer.setAllChecked(true);
	}
	
	/**
	 * 
	 * @author n.hoffmann
	 * @created 12.06.2009
	 * @version 1.0
	 */
	private class MenuPreferenceContentProvider implements IStructuredContentProvider{

		public Object[] getElements(Object inputElement) {			
			return ((Set<?>) inputElement).toArray(new DefinedTermBase[0]);
		}

		public void dispose() {}

		public void inputChanged(Viewer viewer, Object oldInput,
				Object newInput) {}
		
	};
	
	
	
	/**
	 * 
	 * @author n.hoffmann
	 * @created 12.06.2009
	 * @version 1.0
	 */
	class MenuPreferenceLabelProvider extends LabelProvider implements ITableLabelProvider{

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			return ((DefinedTermBase)element).getLabel();
		}
		
	}

	/**
	 * @return the tableViewer
	 */
	public CheckboxTableViewer getTableViewer() {
		return tableViewer;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performOk()
	 */
	@Override
	public boolean performOk() {
		List<T> preferredTerms = new ArrayList<T>();
		for (Object element : getTableViewer().getCheckedElements()){
			preferredTerms.add((T) element);
		}
		
		PreferencesUtil.setPreferredTerms(preferredTerms, this.vocabulary);
		
		PreferencesUtil.firePreferencesChanged(this.getClass());
		
		return true;
	}
	
}